  /*
      Portals - Implicit Grant
  
      Token Endpoint
  
      Description:
          * getAuthenticationToken function is used to fetch an ID token using the Token endpoint in Portals.
          * getAuthenticationToken is added to window.auth using an IIFE.
          * After the IIFE gets executed, the token can be fetched anytime using window.auth.getAuthenticationToken(callback);
          * getAuthenticationToken accepts a callback function to handle the token returned from Portals.
          * Token is returned only for authenticated users.
          * ImplicitGrantForceLogin cookie key is used to decide whether or not to redirect user to login page based on the Token endpoint's response.
  */
  // Part 1: Add getAuthenticationToken to window.auth

$(document).ready(function () {
  (function (auth, $) {
    "use strict";

    // Force Login for Anonymous users. Below line can be commented if force login is not needed.
    $.cookie("ImplicitGrantForceLogin", 1);

    let clientId = `WOI-SelfServicePortal-${ENV.TENANT_ID}`;

    auth.getAuthenticationToken = async function () {
      try {
        const token = await new Promise((resolve, reject) => {
          $.ajax({
            type: 'POST',
            url: `/_services/auth/token?client_id=${clientId}`,
            cache: false,
            success: function (token, status, jqXHR) {
              handleGetAuthenticationTokenSuccess(token, status, jqXHR, resolve, reject);
            },
            error: function (jqXHR) {
              reject(jqXHR);
            }
          });
        });
        return token;
      } catch (error) {
        throw error;
      }
    };

    function handleGetAuthenticationTokenSuccess(token, status, jqXHR, resolve, reject) {
      var jsonResult = JSON.parse(jqXHR.getResponseHeader('X-Responded-JSON'));
      if (jsonResult && jsonResult.status == 401) {
        var forceLogin = Number($.cookie("ImplicitGrantForceLogin")) === 1;
        if (forceLogin) {
          // If the user is not logged in, redirect to the login page
          redirectToLogin();
          reject(new Error('User needs to log in.'));
        } else {
          // Resolve with the error message
          resolve(jsonResult);
        }
      } else {
        // Pass the token to the promise's resolve function
        resolve(token);
      }
    }

    function redirectToLogin() {
      var redirectUrl = window.location;
      var loginUrl = window.location.origin + '/SignIn?returnUrl=' + encodeURIComponent(redirectUrl);
      window.location = loginUrl;
    }
  }(window.auth || (window.auth = {}), window.jQuery));

  (function(webapi, $){
    function safeAjax(ajaxOptions) {
      var deferredAjax = $.Deferred();

      shell.getTokenDeferred().done(function (token) {
        // add headers for AJAX
        if (!ajaxOptions.headers) {
          $.extend(ajaxOptions, {
            headers: {
              "__RequestVerificationToken": token
            }
          }); 
        } else {
          ajaxOptions.headers["__RequestVerificationToken"] = token;d
        }
        $.ajax(ajaxOptions)
          .done(function(data, textStatus, jqXHR) {
            validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
        }).fail(deferredAjax.reject); //AJAX
      }).fail(function () {
          deferredAjax.rejectWith(this, arguments); // on token failure pass the token AJAX and args
      });

      return deferredAjax.promise();	
    }
    webapi.safeAjax = safeAjax;
  })(window.webapi = window.webapi || {}, jQuery)
});